@@ -37,7 +37,7 @@
          eat_crnl/2, get_chunk_num/2, get_chunk/4,
          list_to_uue/1, uue_to_list/1, printversion/0,
          strip_spaces/1, strip_spaces/2,
-         month/1, mk2/1, 
+         month/1, mk2/1, home/0,
          arg_rewrite/1, to_lowerchar/1, to_lower/1, funreverse/2, is_prefix/2,
          split_sep/2, accepts_gzip/2, upto_char/2, deepmap/2,
          ticker/2, ticker/3]).
@@ -59,6 +59,7 @@
          dcc/2]).
 
 -export([make_allow_header/0,
+         make_allow_header/1,
          make_server_header/0,
          make_last_modified_header/1,
          make_location_header/1,
@@ -274,7 +275,9 @@ setup_sconf(DocRoot, D, SL) ->
            allowed_scripts = lkup(allowed_scripts, SL, 
                                   D#sconf.allowed_scripts),
            revproxy = lkup(revproxy, SL, 
-                           D#sconf.revproxy)}.
+                           D#sconf.revproxy),
+           soptions = lkup(soptions, SL,
+                           D#sconf.soptions)}.
 
 set_sc_flags([{access_log, Bool}|T], Flags) ->
     set_sc_flags(T, flag(Flags, ?SC_ACCESS_LOG, Bool));
@@ -311,12 +314,15 @@ hup(Sock) ->
 
 dohup(Sock) ->
     Env = yaws_sup:get_app_args(),
-    Res = (catch case yaws_config:load(Env) of
-                     {ok, Gconf, Sconfs} ->
-                         yaws_api:setconf(Gconf, Sconfs);
-                     Err ->
-                         Err
-                 end),
+    Res = try yaws_config:load(Env) of
+              {ok, Gconf, Sconfs} ->
+                  yaws_api:setconf(Gconf, Sconfs);
+              Err ->
+                  Err
+          catch
+              _:X ->
+                  X
+          end,
     gen_event:notify(yaws_event_manager, {yaws_hupped, Res}),
     gen_event:notify(yaws_log, {yaws_hupped, Res}),
     gen_tcp:send(Sock, io_lib:format("hupped: ~p~n", [Res])),
@@ -1259,13 +1265,24 @@ dcc(Req, Headers) ->
 %%
 
 make_allow_header() ->
-    HasDav = ?sc_has_dav(get(sc)),
-    ["Allow: GET, POST, OPTIONS, HEAD",
-     if HasDav == true ->
-             ", PUT, PROPFIND, MKCOL, MOVE, COPY\r\n";
-        true ->
-             "\r\n"
-     end].
+    make_allow_header([]).
+make_allow_header(Options) ->
+    case Options of
+        [] ->
+            HasDav = ?sc_has_dav(get(sc)),
+            ["Allow: GET, POST, OPTIONS, HEAD",
+             if HasDav == true ->
+                     ", PUT, PROPFIND, MKCOL, MOVE, COPY\r\n";
+                true ->
+                     "\r\n"
+             end];
+        _ ->
+            ["Allow: ",
+             lists:foldl(fun(M, "") -> atom_to_list(M);
+                            (M, Acc) -> atom_to_list(M) ++ ", " ++ Acc end,
+                         "", lists:reverse(Options)),
+             "\r\n"]
+    end.
 make_server_header() ->
     HasDav = ?sc_has_dav(get(sc)),
     ["Server: Yaws/", yaws_generated:version(), " Yet Another Web Server\r\n" |
@@ -1585,7 +1602,7 @@ erase_header(location) ->
 getuid() ->
     case os:type() of
         {win32, _} ->
-            {ok, "XXX"};
+            {ok, "0"};
         _ ->
             load_setuid_drv(),
             P = open_port({spawn, "setuid_drv g"},[]),
@@ -1594,14 +1611,21 @@ getuid() ->
                     {ok, IntList}
             end
     end.
+
 user_to_home(User) ->
-    load_setuid_drv(),
-    P = open_port({spawn, "setuid_drv " ++ [$h|User]}, []),
-    receive
-        {P, {data, "ok " ++ Home}} ->
-            Home
+    case os:type() of
+        {win32, _} ->
+            ".";
+        _ ->
+            load_setuid_drv(),
+            P = open_port({spawn, "setuid_drv " ++ [$h|User]}, []),
+            receive
+                {P, {data, "ok " ++ Home}} ->
+                    Home
+            end
     end.
 
+
 uid_to_name(Uid) ->
     load_setuid_drv(),
     P = open_port({spawn, "setuid_drv " ++ [$n|integer_to_list(Uid)]}, []),
@@ -1615,6 +1639,7 @@ load_setuid_drv() ->
                true ->
                    filename:dirname(code:which(?MODULE)) ++ "/../priv/lib";
                false ->
+                   %% ignore dialyzer on this one
                    PrivDir = code:priv_dir(yaws),
                    filename:join(PrivDir,"lib")
            end,
@@ -1748,9 +1773,7 @@ http_get_headers(CliSock, SSL) ->
         tuple(Res) ->
             {Request, Headers} = Res,
             ReqStr = yaws_api:reformat_request(Request),
-            HStr = lists:map(
-                     fun(H) -> [H, "\r\n"] end,
-                     yaws_api:reformat_header(Headers)),
+            HStr = headers_to_str(Headers),
             yaws_log:trace_traffic(from_client, 
                                    ?F("~n~s~n~s~n",[ReqStr, HStr])),
             Res;
@@ -1780,7 +1803,7 @@ do_http_get_headers(CliSock, SSL) ->
         closed -> 
             closed;
         R -> 
-            H = http_collect_headers(CliSock, R,  #headers{}, SSL),
+            H = http_collect_headers(CliSock, R,  #headers{}, SSL, 0),
             {R, H}
     end.
 
@@ -1807,56 +1830,70 @@ http_recv_request(CliSock, SSL) ->
 
 
 
-http_collect_headers(CliSock, Req, H, SSL) ->
-    case do_recv(CliSock, 0, SSL) of
+http_collect_headers(CliSock, Req, H, SSL, Count) when Count < 1000 ->
+    Recv = do_recv(CliSock, 0, SSL),
+    case Recv of
         {ok, {http_header,  _Num, 'Host', _, Host}} ->
-            http_collect_headers(CliSock, Req, H#headers{host = Host},SSL);
+            http_collect_headers(CliSock, Req, H#headers{host = Host},
+                                 SSL, Count+1);
         {ok, {http_header, _Num, 'Connection', _, Conn}} ->
             http_collect_headers(CliSock, Req, 
-                                 H#headers{connection = Conn},SSL);
+                                 H#headers{connection = Conn},SSL, Count+1);
         {ok, {http_header, _Num, 'Accept', _, Accept}} ->
-            http_collect_headers(CliSock, Req, H#headers{accept = Accept},SSL);
+            http_collect_headers(CliSock, Req, H#headers{accept = Accept},
+                                 SSL, Count+1);
         {ok, {http_header, _Num, 'If-Modified-Since', _, X}} ->
             http_collect_headers(CliSock, Req,  
-                                 H#headers{if_modified_since = X},SSL);
+                                 H#headers{if_modified_since = X},SSL, Count+1);
         {ok, {http_header, _Num, 'If-Match', _, X}} ->
-            http_collect_headers(CliSock, Req, H#headers{if_match = X},SSL);
+            http_collect_headers(CliSock, Req, H#headers{if_match = X},
+                                 SSL, Count+1);
         {ok, {http_header, _Num, 'If-None-Match', _, X}} ->
             http_collect_headers(CliSock, Req, 
-                                 H#headers{if_none_match = X},SSL);
+                                 H#headers{if_none_match = X},SSL, Count+1);
         {ok, {http_header, _Num, 'If-Range', _, X}} ->
-            http_collect_headers(CliSock, Req, H#headers{if_range = X},SSL);
+            http_collect_headers(CliSock, Req, H#headers{if_range = X},
+                                 SSL, Count+1);
         {ok, {http_header, _Num, 'If-Unmodified-Since', _, X}} ->
             http_collect_headers(CliSock, Req,  
-                                 H#headers{if_unmodified_since = X},SSL);
+                                 H#headers{if_unmodified_since = X},SSL, 
+                                 Count+1);
         {ok, {http_header, _Num, 'Range', _, X}} ->
-            http_collect_headers(CliSock, Req, H#headers{range = X},SSL);
+            http_collect_headers(CliSock, Req, H#headers{range = X},
+                                 SSL, Count+1);
         {ok, {http_header, _Num, 'Referer',_, X}} ->
-            http_collect_headers(CliSock, Req, H#headers{referer = X},SSL);
+            http_collect_headers(CliSock, Req, H#headers{referer = X},
+                                 SSL, Count+1);
         {ok, {http_header, _Num, 'User-Agent', _, X}} ->
-            http_collect_headers(CliSock, Req, H#headers{user_agent = X},SSL);
+            http_collect_headers(CliSock, Req, H#headers{user_agent = X},
+                                 SSL, Count+1);
         {ok, {http_header, _Num, 'Accept-Ranges', _, X}} ->
             http_collect_headers(CliSock, Req, 
-                                 H#headers{accept_ranges = X},SSL);
+                                 H#headers{accept_ranges = X},SSL, Count+1);
         {ok, {http_header, _Num, 'Cookie', _, X}} ->
             http_collect_headers(CliSock, Req,  
-                                 H#headers{cookie = [X|H#headers.cookie]},SSL);
+                                 H#headers{cookie = [X|H#headers.cookie]},
+                                 SSL, Count+1);
         {ok, {http_header, _Num, 'Keep-Alive', _, X}} ->
-            http_collect_headers(CliSock, Req, H#headers{keep_alive = X},SSL);
+            http_collect_headers(CliSock, Req, H#headers{keep_alive = X},
+                                 SSL, Count+1);
         {ok, {http_header, _Num, 'Content-Length', _, X}} ->
             http_collect_headers(CliSock, Req, 
-                                 H#headers{content_length = X},SSL);
+                                 H#headers{content_length = X},SSL, 
+                                 Count+1);
         {ok, {http_header, _Num, 'Content-Type', _, X}} ->
             http_collect_headers(CliSock, Req, 
-                                 H#headers{content_type = X},SSL);
+                                 H#headers{content_type = X},SSL, Count+1);
         {ok, {http_header, _Num, 'Transfer-Encoding', _, X}} ->
             http_collect_headers(CliSock, Req, 
-                                 H#headers{transfer_encoding=X},SSL);
+                                 H#headers{transfer_encoding=X},SSL, Count+1);
         {ok, {http_header, _Num, 'Location', _, X}} ->
-            http_collect_headers(CliSock, Req, H#headers{location=X},SSL);
+            http_collect_headers(CliSock, Req, H#headers{location=X},
+                                 SSL, Count+1);
         {ok, {http_header, _Num, 'Authorization', _, X}} ->
             http_collect_headers(CliSock, Req,  
-                                 H#headers{authorization = parse_auth(X)},SSL);
+                                 H#headers{authorization = parse_auth(X)},
+                                 SSL, Count+1);
 
         {ok, http_eoh} ->
             H;
@@ -1865,19 +1902,23 @@ http_collect_headers(CliSock, Req, H, SSL) ->
         %% bad (typically test script) clients
 
         {error, {http_error, "\r\n"}} ->
-            http_collect_headers(CliSock, Req, H,SSL);
+            http_collect_headers(CliSock, Req, H,SSL, Count+1);
         {error, {http_error, "\n"}} ->
-            http_collect_headers(CliSock, Req, H,SSL);
+            http_collect_headers(CliSock, Req, H,SSL, Count+1);
 
         %% auxilliary headers we don't have builtin support for
         {ok, X} ->
             ?Debug("OTHER header ~p~n", [X]),
             http_collect_headers(CliSock, Req,  
-                                 H#headers{other=[X|H#headers.other]},SSL);
+                                 H#headers{other=[X|H#headers.other]},
+                                 SSL, Count+1);
         _Err ->
             exit(normal)
 
-    end.
+    end;
+http_collect_headers(_CliSock, _Req, _H, _SSL, _Count)  ->
+    error_logger:format("Max num headers - DOS attack closing\n", []),
+    exit(normal).
 
 
 
@@ -2084,9 +2125,19 @@ tmpdir() ->
                     PathTEMP
             end;
         _ ->
-            filename:join([os:getenv("HOME"), ".yaws"])
+	    filename:join([home(), ".yaws"])
     end.
 
+%% This feature is useable together with 
+%% privbind and authbind on linux
+
+home() ->
+    case os:getenv("YAWSHOME") of
+	false ->
+	    os:getenv("HOME");
+	DIR ->
+	    DIR
+    end.
 
 id_dir(Id) ->
     filename:join([tmpdir(), "yaws", to_list(Id)]).
