@@ -33,7 +33,7 @@
 -export([get_line/1, mime_type/1]).
 -export([stream_chunk_deliver/2, stream_chunk_deliver_blocking/2,
          stream_chunk_end/1]).
--export([new_cookie_session/1, new_cookie_session/2,
+-export([new_cookie_session/1, new_cookie_session/2, new_cookie_session/3, 
          cookieval_to_opaque/1, request_url/1,
          print_cookie_sessions/0,
          replace_cookie_session/2, delete_cookie_session/1]).
@@ -57,7 +57,7 @@
          postvar/2, queryvar/2, getvar/2]).
 
 -export([binding/1,binding_exists/1,
-         dir_listing/1, dir_listing/2]).
+         dir_listing/1, dir_listing/2, redirect_self/1]).
 
 
 -import(lists, [map/2, flatten/1, reverse/1]).
@@ -90,8 +90,8 @@ parse_post(Arg) ->
             end;
         Other ->
             error_logger:error_msg(
-              "ERROR: Can't parse post body for ~p requests",
-              [Other]),
+              "ERROR: Can't parse post body for ~p requests: URL: ~p",
+              [Other, Arg#arg.fullpath]),
             []
     end.
 
@@ -230,8 +230,10 @@ parse_arg_value([$\\,$"|Line], Key, Value, Quote, Begun) ->
     parse_arg_value(Line, Key, [$"|Value], Quote, Begun);
 parse_arg_value([$"|Line], Key, Value, false, _) ->
     parse_arg_value(Line, Key, Value, true, true);
-                 parse_arg_value([$"|Line], Key, Value, true, _) ->
-    make_parse_line_reply(Key, Value, Line);
+parse_arg_value([$"], Key, Value, true, _) ->
+    make_parse_line_reply(Key, Value, []);
+parse_arg_value([$",$;|Line], Key, Value, true, _) ->
+    make_parse_line_reply(Key, Value, [$;|Line]);
 parse_arg_value([$;|Line], Key, Value, false, _) ->
     make_parse_line_reply(Key, Value, [$;|Line]);
 parse_arg_value([$ |Line], Key, Value, false, true) ->
@@ -848,6 +850,9 @@ new_cookie_session(Opaque) ->
 new_cookie_session(Opaque, TTL) ->
     yaws_session_server:new_session(Opaque, TTL).
 
+new_cookie_session(Opaque, TTL, Cleanup) ->
+    yaws_session_server:new_session(Opaque, TTL, Cleanup).
+
 %% as returned in #ysession.cookie
 cookieval_to_opaque(CookieVal) ->
     yaws_session_server:cookieval_to_opaque(CookieVal).
@@ -1015,7 +1020,12 @@ reformat_header(H) ->
 
 
 reformat_request(Req) ->
-    {abs_path, Path} = Req#http_request.path,
+    Path = case Req#http_request.path of
+	       {abs_path, AbsPath} ->
+		   AbsPath;
+	       {absoluteURI, _Scheme, _Host0, _Port, RawPath} ->
+		   RawPath
+	   end,
     {Maj,Min} = Req#http_request.version,
     [yaws:to_list(Req#http_request.method), " ", Path," HTTP/",
      integer_to_list(Maj),".", integer_to_list(Min)].
@@ -1515,8 +1525,8 @@ parse_set_cookie_key([C|T], Acc) ->
 
 parse_set_cookie_value([$"|T]) ->
     parse_quoted(T,[]);
-                        parse_set_cookie_value(T) ->
-                               parse_set_cookie_value(T,[]).
+parse_set_cookie_value(T) ->
+    parse_set_cookie_value(T,[]).
 
 parse_set_cookie_value([],Acc) ->
     {lists:reverse(Acc), false, []};
@@ -1529,12 +1539,11 @@ parse_quoted([], Acc) ->
     {lists:reverse(Acc), true, []};
 parse_quoted([$"|T], Acc) ->
     {lists:reverse(Acc), true, T};
-              parse_quoted([$\\,C|T], Acc) ->    
-                     parse_quoted(T,[C,$\\|Acc]);
-              parse_quoted([C|T], Acc) ->
-                     parse_quoted(T,[C|Acc]).
-
-                                                %%
+parse_quoted([$\\,C|T], Acc) ->    
+    parse_quoted(T,[C,$\\|Acc]);
+parse_quoted([C|T], Acc) ->
+    parse_quoted(T,[C|Acc]).
+%%
 
 add_set_cookie(C, Key, Value, Quoted) when C#setcookie.key==undefined ->
     C#setcookie{key=Key,value=Value,quoted=Quoted};
@@ -1660,6 +1669,16 @@ binding_exists(Key) ->
         _ -> true
     end.
 
+%% split inputstring at first occurrence of Char
+split_at(String, Char) ->
+    split_at(String, Char, []).
+split_at([H|T], H, Ack) ->
+    {lists:reverse(Ack), T};
+split_at([H|T], Char, Ack) ->
+    split_at(T, Char, [H|Ack]);
+split_at([], _Char, Ack) ->
+    {[], lists:reverse(Ack)}.
+
 
 %% Return the parsed url that the client requested.
 request_url(ARG) ->
@@ -1667,12 +1686,7 @@ request_url(ARG) ->
     Headers = ARG#arg.headers,
     {abs_path, Path} = (ARG#arg.req)#http_request.path,
     DecPath = url_decode(Path),
-    {P,Q} = case string:tokens(DecPath, "?") of
-                [P0] ->
-                    {P0, []};
-                [P0,Q0] ->
-                    {P0,Q0}
-            end,
+    {P,Q} = split_at(DecPath, $?),
     #url{scheme = case SC#sconf.ssl of
                       undefined ->
                           "http";
@@ -1719,6 +1733,22 @@ sanitize_file_name([]) ->
 %% than /etc/yaws.conf, for example from a database
 %% this code is also called by the server -h hup code
 setconf(GC0, Groups0) ->
+    setconf(GC0, Groups0, true).
+setconf(GC0, Groups0, CheckCertsChanged) ->
+    CertsChanged = if CheckCertsChanged == true ->
+                           lists:member(yes,gen_server:call(
+                                              yaws_server, 
+                                              check_certs, infinity));
+                      true ->
+                           false
+                   end,
+    if
+        CertsChanged ->
+            application:stop(ssl),
+            application:start(ssl);
+        true ->
+            ok
+    end,
     {GC, Groups} = yaws_config:verify_upgrade_args(GC0, Groups0),
     {ok, OLDGC, OldGroups} = yaws_api:getconf(),
     case {yaws_config:can_hard_gc(GC, OLDGC),
@@ -1762,3 +1792,39 @@ dir_listing(Arg, RelDir) ->
             %% make sure it's a valid path here
             ok
     end.
+
+%% Returns #redir_self{} record
+redirect_self(A) ->
+    SC = get(sc),
+    {Port, PortStr} = 
+        case {SC#sconf.rmethod, SC#sconf.ssl, SC#sconf.port} of
+            {"https", _, 443} -> {443, ""};
+            {"http", _, 80} -> {80, ""};
+            {_, undefined, 80} -> {80, ""};
+            {_, undefined, Port2} -> 
+                {port, [$:|integer_to_list(Port2)]};
+            {_, _SSL, 443} ->
+                {443, ""};
+            {_, _SSL, Port2} -> 
+                {Port2, [$:|integer_to_list(Port2)]}
+        end,
+    H = A#arg.headers,
+    Host = yaws:redirect_host(get(sc), H#headers.host),
+    {Scheme, SchemeStr} = 
+        case {SC#sconf.ssl,SC#sconf.rmethod} of
+            {_, Method} when list(Method) ->
+                {list_to_atom(Method), Method++"://"};
+            {undefined,_} ->
+                {http, "http://"};
+            {_SSl,_} ->
+                {https, "https://"}
+        end,
+    #redir_self{host = Host,
+                scheme = Scheme,
+                scheme_str = SchemeStr,
+                port = Port,
+                port_str = PortStr}.
+
+
+
+
