@@ -34,7 +34,7 @@ paths() ->
         {ok, "0"} ->    %% root 
             [yaws_generated:etcdir() ++ "/yaws.conf"];
         _ -> %% developer
-            [filename:join([os:getenv("HOME"), "yaws.conf"]),
+            [filename:join([yaws:home(), "yaws.conf"]),
              "./yaws.conf", 
              yaws_generated:etcdir() ++ "/yaws.conf"]
     end.
@@ -46,7 +46,7 @@ paths() ->
 load(E = #env{conf = false}) ->
     case yaws:first(fun(F) -> yaws:exists(F) end, paths()) of
         false ->
-            {error, "Can't find no config file "};
+            {error, "Can't find config file "};
         {ok, _, File} ->
             load(E#env{conf = {file, File}})
     end;
@@ -76,7 +76,7 @@ load(E) ->
                     Err
             end;
         _ ->
-            {error, "Can't open config file" ++ File}
+            {error, "Can't open config file " ++ File}
     end.
 
 
@@ -183,12 +183,12 @@ validate_group(List) ->
             ok
     end,
     %% second all servernames in a group must be unique
-    SN = lists:sort([X#sconf.servername || X <- List]),
+    SN = lists:sort([yaws:to_lower(X#sconf.servername) || X <- List]),
     no_two_same(SN).
 
 no_two_same([H,H|_]) ->
     throw({error, 
-           ?F("To servers in the same group cannot have same name ~p",[H])});
+           ?F("Two servers in the same group cannot have same name ~p",[H])});
 no_two_same([_H|T]) ->
     no_two_same(T);
 no_two_same([]) ->
@@ -299,7 +299,7 @@ fload(FD, globals, GC, _C, Cs, _Lno, eof) ->
 
 fload(FD, _,  _GC, _C, _Cs, Lno, eof) ->
     file:close(FD),
-    {error, ?F("Unexpected end of file at line ~w", [Lno])};
+    {error, ?F("Unexpected end-of-file at line ~w", [Lno])};
 
 fload(FD, globals, GC, C, Cs, Lno, Chars) -> 
     Next = io:get_line(FD, ''),
@@ -325,7 +325,7 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
                                     Err
                             end;
                         _ ->
-                            {error, "Can't open config file" ++ File}
+                            {error, "Can't open config file " ++ File}
                     end;
                 false ->
                     {error, ?F("Expect filename at line ~w", [Lno])}
@@ -364,7 +364,7 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
                                                                    ?Debug("FLOAD: ~p", [R]),
                                                                    R;
                                                                _ ->
-                                                                   {error, "Can't open config file" ++
+                                                                   {error, "Can't open config file " ++
                                                                     File}
                                                            end;
                                                        false ->
@@ -417,13 +417,14 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
 
         ["ebin_dir", '=', Ebindir] ->
             Dir = filename:absname(Ebindir),
-            case is_dir(Dir) of
+            case warn_dir("ebin_dir", Dir) of
                 true ->
                     fload(FD, globals, GC#gconf{ebin_dir = 
                                                 [Dir|GC#gconf.ebin_dir]},
                           C, Cs, Lno+1, Next);
                 false ->
-                    {error, ?F("Expect directory at line ~w (ebin_dir: ~s)", [Lno, Dir])}
+                    fload(FD, globals, GC, C, Cs, Lno+1, Next)
+	    
             end;
 
         ["runmod", '=', Mod0] ->
@@ -470,13 +471,14 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
 
         ["include_dir", '=', Incdir] ->
             Dir = filename:absname(Incdir),
-            case is_dir(Dir) of
+            case warn_dir("include_dir", Dir) of
                 true ->
                     fload(FD, globals, GC#gconf{include_dir=
                                                 [Dir|GC#gconf.include_dir]},
                           C, Cs, Lno+1, Next);
                 false ->
-                    {error, ?F("Expect directory at line ~w (include_dir: ~s)", [Lno, Dir])}
+                    fload(FD, globals, GC, C, Cs, Lno+1, Next)
+
             end;
 
         ["mnesia_dir", '=', Mnesiadir] ->
@@ -487,7 +489,7 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
                     fload(FD, globals, GC#gconf{mnesia_dir = Dir},
                           C, Cs, Lno+1, Next);
                 false ->
-                    {error, ?F("Expect directory at line ~w", [Lno])}
+                    {error, ?F("Expect mnesia directory at line ~w", [Lno])}
             end;
 
         ["tmpdir", '=', _TmpDir] ->
@@ -713,6 +715,21 @@ fload(FD, server, GC, C, Cs, Lno, Chars) ->
                     C2 = C#sconf{listen = Addr},
                     fload(FD, server, GC, C2, Cs, Lno+1, Next)
             end;
+        ["listen_backlog", '=', Val] ->
+            case (catch list_to_integer(Val)) of
+                B when integer(B) ->
+                    C2 = case proplists:get_value(listen_opts, C#sconf.soptions) of
+                             undefined ->
+                                 C#sconf{soptions = [{listen_opts, [{backlog, B}]} |
+                                                     C#sconf.soptions]};
+                             Opts ->
+                                 C#sconf{soptions = [{listen_opts, [{backlog, B} | Opts]} |
+                                                     C#sconf.soptions]}
+                         end,
+                    fload(FD, server, GC, C2, Cs, Lno+1, Next);
+                _ ->
+                    {error, ?F("Expect integer at line ~w", [Lno])}
+            end;
         ["servername", '=', Name] ->
             C2 = ?sc_set_add_port((C#sconf{servername = Name}),false),
             fload(FD, server, GC, C2, Cs, Lno+1, Next);
@@ -984,6 +1001,16 @@ fload(FD, opaque, GC, C, Cs, Lno, Chars) ->
         [Key, '=', Value] ->
             C2 = C#sconf{opaque = [{Key,Value} | C#sconf.opaque]},
             fload(FD, opaque, GC, C2, Cs, Lno+1, Next);
+        [Key, '='| Value] ->
+            String_value = lists:flatten(
+                             lists:map(
+                               fun(Item) when is_atom(Item) ->
+                                       atom_to_list(Item);
+                                  (Item) ->
+                                       Item
+                               end, Value)),
+            C2 = C#sconf{opaque = [{Key, String_value} | C#sconf.opaque]},
+            fload(FD, opaque, GC, C2, Cs, Lno+1, Next);
         [H|T] ->
             {error, ?F("Unexpected input ~p at line ~w", [[H|T], Lno])}
     end.
@@ -1071,6 +1098,16 @@ is_bool(_) ->
     false.
 
 
+warn_dir(Type, Dir) ->
+    case is_dir(Dir) of
+	true ->
+	    true;
+	false ->
+	    error_logger:format("Config Warning: Directory ~s for ~s doesn't exist~n",
+				[Dir, Type]),
+	    false
+    end.
+
 is_dir(Val) ->
     case file:read_file_info(Val) of
         {ok, FI} when FI#file_info.type == directory ->
@@ -1318,7 +1355,8 @@ eq_sconfs(S1,S2) ->
      S1#sconf.opaque == S2#sconf.opaque andalso
      S1#sconf.start_mod == S2#sconf.start_mod andalso
      S1#sconf.allowed_scripts == S2#sconf.allowed_scripts andalso
-     S1#sconf.revproxy == S2#sconf.revproxy).
+     S1#sconf.revproxy == S2#sconf.revproxy andalso
+     S1#sconf.soptions == S2#sconf.soptions).
 
 
 
@@ -1509,9 +1547,6 @@ is_list_of_scs([]) ->
 is_list_of_scs(_) ->
     false.
 
-
-
-
 add_sconf(SC) ->
     ok= gen_server:call(yaws_server, {add_sconf, SC}, infinity),
     ok = gen_server:call(yaws_log, {soft_add_sc, SC}, infinity).
