@@ -37,9 +37,7 @@ run(GC) ->
     %% with the same sid.
     case connect(GC#gconf.id) of
         {ok, Sock, _Key} ->
-            %% Not good, let's get some sys info
-            %% from that system so we can produce a good error
-            %% message
+            %% Not good, 
             gen_tcp:close(Sock),
             e("There is already a yaws system running with the same ~n"
               " id <~p> on this computer and this user, ~n"
@@ -55,17 +53,25 @@ run(GC) ->
     end.
 
 rand() ->
-    try 
-        crypto:start(),
-        crypto:rand_uniform(0, 1 bsl 64)
-    catch
-        _ ->
-            {A1, A2, A3}=now(),
+    case os:type() of
+        {win32, _} ->
+             {A1, A2, A3}=now(),
             random:seed(A1, A2, A3),
-            random:uniform(1 bsl 64)
+            random:uniform(1 bsl 64);
+        _ ->
+            try 
+                crypto:start(),
+                crypto:rand_uniform(0, 1 bsl 64)
+            catch
+                _ ->
+                    {A1, A2, A3}=now(),
+                    random:seed(A1, A2, A3),
+                    random:uniform(1 bsl 64)
+            end
     end.
 
 
+
 ctl_args() ->
     [{packet, 2},
      {active, false},
@@ -220,7 +226,7 @@ f(Fmt, As) ->
 
 
 a_id(Sock) ->
-    ID = gen_server:call(yaws_server, id, [], infinity),
+    ID = gen_server:call(yaws_server, id, infinity),
     gen_tcp:send(Sock, ID),
     ok.
 
