@@ -396,9 +396,9 @@ handle_info(minute10, State) ->
         {ok, FI} when  State#state.log_wrap_size > 0, 
         FI#file_info.size > State#state.log_wrap_size, 
         State#state.copy_errlog == true ->
-            gen_event:call(error_logger, yaws_log_file_h, wrap);
+            gen_event:call(error_logger, yaws_log_file_h, wrap, infinity);
         {error,enoent} ->
-            gen_event:call(error_logger, yaws_log_file_h, reopen);
+            gen_event:call(error_logger, yaws_log_file_h, reopen, infinity);
         _ ->
             ok
     end,
@@ -440,6 +440,7 @@ wrap(AL, State) ->
             %% Logfile disapeared, 
             error_logger:format("Logfile ~p disapeared - we reopen it",
                                 [AL#alog.filename]),
+	    file:close(AL#alog.fd),
             {ok, Fd2} = file:open(AL#alog.filename, [write, raw]),
             AL#alog{fd = Fd2}
     end.
