@@ -318,8 +318,8 @@ get_url_file("http://"++_ = URL) ->
 	{ok,{{_HTTP,RC,Emsg}, _Headers, _Body}} -> 
 	    error_logger:error_msg("~p: http-request got: ~p~n", [?MODULE, {RC, Emsg}]),
 	    {error, "failed to retrieve: "++URL};
-	_ -> 
-	    error_logger:error_msg("~p: http-request failed~n", []),
+	{error, Reason} -> 
+	    error_logger:error_msg("~p: http-request failed: ~p~n", [?MODULE, Reason]),
 	    {error, "failed to retrieve: "++URL}
     end;
 get_url_file("file://"++Fname) ->
@@ -352,7 +352,7 @@ inets_request(URL, SoapAction, Request, Options, Headers) ->
                       {URL,NewHeaders,
                        "text/xml; charset=utf-8",
                        Request},
-                      [{http_timeout,20000}],
+                      [{timeout,20000}],
                       [{sync, true}, {full_result, true}, {body_format, string}]) of
         {ok,{{_HTTP,200,_OK},ResponseHeaders,ResponseBody}} ->
             {ok, 200, ResponseHeaders, ResponseBody};
@@ -551,7 +551,7 @@ searchPorts(BindingName, [Port | Tail], Acc) ->
     #port{binding = #qname{localPart = BindingName}} ->
       searchPorts(BindingName, Tail, [Port | Acc]);
     _ -> 
-      searchPorts(BindingName, Tail)
+      searchPorts(BindingName, Tail, Acc)
   end.
 
 
